#include <CL/sycl.hpp>
#include <iostream>
#include <math.h>
#include <array>
using namespace sycl;

constexpr int N = 42;

int main() {

  std::array<float,N> host_array;
  for (int i = 0; i < N; i++)
    host_array[i] = i;

  queue Q{host_selector{}};

  std::cout << "Dispositivo: " << Q.get_device().get_info<info::device::name>()   << "\n";
  std::cout << "             " << Q.get_device().get_info<info::device::vendor>() << "\n";

  auto *device_array = malloc_device<float>(N, Q);

  Q.submit([&](handler& h) {
    // copy hostArray to deviceArray
    h.memcpy(device_array, &host_array[0], N * sizeof(float));
  });
  Q.wait(); 

  Q.submit([&](handler& h) {
    h.parallel_for(N, [=](id<1> i) {
      device_array[i] = sqrt(device_array[i]);
    });
  });
  Q.wait();

  Q.submit([&](handler& h) {
    // copy deviceArray back to hostArray
    h.memcpy(&host_array[0], device_array, N * sizeof(float));
  });
  Q.wait();

  for(int i = 0; i < N; ++i) 
    std::cout << '[' << host_array[i] << ']'; std::cout << std::endl;

  free(device_array, Q);
  return 0;
}
