#include <CL/sycl.hpp>
#include <iostream>
#include <array>
using namespace sycl;
constexpr int N = 64; // N deve ser potência de 2 e N%num_groups=0.
constexpr int num_groups = 8; 

int main() {

  std::array<float,num_groups> output;
  std::array<float,N> input;
  for (int i = 0; i < N; i++)
    input[i] = i;

  queue Q{cpu_selector{}};

  float sum = 0.;
  //for(int i = 0; i < N; ++i) {
  //  sum += input[i]; 
  //  std::cout << '[' << input[i] << ']'; std::cout << std::endl;
  //}
  //std::cout << '[' << sum << ']'; std::cout << std::endl;

  range global_range{N};
  range local_range{N/num_groups};
  
  {
     buffer A{input};
     buffer B{output};
  
     Q.submit([&](handler &h) {
       accessor in{A, h};  
       accessor out{B, h};  

       auto sum_partial = accessor<float, 1, access::mode::read_write, access::target::local>(local_range, h);
     
       h.parallel_for(nd_range{global_range, local_range}, [=] (nd_item<1> id) {
         int global_id = id.get_global_id(0);
         int local_id = id.get_local_id(0);
         int group_id = id.get_group(0);
  
         sum_partial[local_id] = in[global_id];
         
         for (int s = id.get_local_range(0)/2; s > 0; s >>= 1) {    

           id.barrier(sycl::access::fence_space::local_space);

           if (local_id < s) { 
             sum_partial[ local_id ] += sum_partial[ local_id + s ]; 
           } 
         }
     
         if(local_id == 0) { 
           out[group_id] = sum_partial[0]; 
         }
       });
     });
  }

  for(int i = 0; i < N; ++i) 
    std::cout << '[' << input[i] << ']'; std::cout << std::endl;

  sum = 0;
  for(int i = 0; i < num_groups; ++i) 
    sum += output[i]; 
  std::cout << '[' << sum << ']'; std::cout << std::endl;

  return 0;
}

